import {app, BrowserWindow, dialog, ipcMain, powerSaveBlocker, screen} from "electron";
import * as path from "path";
import AppUpdater from "./updater";
const crypto = require('crypto');

const setupPug = require('electron-pug');
const packageJson = require("../package.json");
const Store = require('electron-store');


const store = new Store();


process.on("uncaughtException", (err) => {
    console.log("Uncaught Exception")
    console.error(err);
    const messageBoxOptions = {
        type: "error",
        title: "Shit - There was an error!",
        message: err.message
    };
    dialog.showMessageBoxSync(messageBoxOptions);
});

const updater = new AppUpdater();
let pug;


const operatingSystem = process.platform;
console.log("Starting TryCrypt v" + packageJson.version);
console.log("Detected operating system is " + operatingSystem);

let mainWindow: BrowserWindow;


function sleep(ms: number) {
    return new Promise((resolve) => {
        setTimeout(() => resolve(null), ms);
    });
}


/**
 * Main window
 */
function createWindow() {
    return new Promise((resolve) => {
        // Create the browser window.
        mainWindow = new BrowserWindow({
            webPreferences: {
                preload: path.join(__dirname, "preload.js"),
                contextIsolation: true,
                nodeIntegration: true,
            },
            width: 1000,
            height: 600,
            alwaysOnTop: false,
            autoHideMenuBar: false,
            title: "TryCrypt",
            show: true,
        });

        // and load the index.html of the app.
        mainWindow.loadFile(path.join(__dirname, "../static/index.pug"));

        mainWindow.on("ready-to-show", (e: Event) => {
            resolve(true);
        });

        mainWindow.on("closed", () => {
            console.log("Closing app...");
            app.exit(0);
        });
    });
}


/**
 * App is ready (Electron has booted up!)
 */
app.on("ready", async () => {
    try {
        pug = await setupPug({pretty: true});
        pug.on('error', (err: string) => console.error('render error', err));

        let updateAvailable = await updater.checkForUpdates();
        if (updateAvailable) {
            await updater.downloadAndInstallUpdate();
            const messageBoxOptions = {
                type: "info",
                title: "Update!",
                message: "An update of this application will be installed now..."
            };
            dialog.showMessageBoxSync(messageBoxOptions);
            return;
        }

        await createWindow();

    } catch (e) {
        app.relaunch();
        app.exit();
    }
});


////// IPC HANDLES \\\\\\\

function generateRandom(length: number) {
    let result = '';
    const characters = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789';
    const charactersLength = characters.length;
    let counter = 0;
    while (counter < length) {
        result += characters.charAt(Math.floor(Math.random() * charactersLength));
        counter += 1;
    }
    return result;
}

// Relaunch handler
ipcMain.on("relaunch", () => {
    app.relaunch();
    app.exit();
    return;
});


ipcMain.handle("getKey", (e) => {
    console.log("Key requested!");
    return [store.get('key'), store.get('iv')];
});

ipcMain.handle("generateKey", (e) => {
    console.log("Key generating...");
    const key = generateRandom(32);
    const iv = generateRandom(16);
    return [key, iv];
});

ipcMain.on("setKey", (e, key: string, iv: string) => {
    console.log("Got new key!");
    store.set('key', key);
    store.set('iv', iv);
});

ipcMain.handle("encrypt", (e, text) => {
    console.log("Encrypting...");
    const cipher = crypto.createCipheriv('aes-256-cbc', Buffer.from(store.get('key')), Buffer.from(store.get('iv')));
    const encryptedData = cipher.update(text, 'utf8', 'base64') + cipher.final('base64');
    return encryptedData;
});

ipcMain.handle("decrypt", (e, text) => {
    console.log("Decrypting...");
    const decipher = crypto.createDecipheriv('aes-256-cbc', Buffer.from(store.get('key')), Buffer.from(store.get('iv')));
    let decrypted = decipher.update(text, 'base64', 'utf8');
    decrypted += decipher.final('utf8');
    return decrypted.toString();
});