/*
 * Copyright (c) Tobias Hopp, 2022.
 * This program is copyright protected.
 * Copying, distributing or using source code is prohibited.
 * Any distribution or illegal copying will be prosecuted.
 *
 * You may not use this sourcecode.
 *
 */

import {autoUpdater} from "electron-updater";

export default class AppUpdater {
    constructor() {

        //log.transports.file.level = "debug"
        autoUpdater.autoDownload = false;
        autoUpdater.allowDowngrade = false;
        autoUpdater.setFeedURL("https://infoscreen.iif.li/releases/");
    }

    public checkForUpdates(): Promise<boolean> {
        return new Promise((resolve, reject) => {
            autoUpdater.on('update-available', (ev) => {
                resolve(true);
            })
            autoUpdater.on('update-not-available', (ev) => {
                resolve(false);
            })

            autoUpdater.on("update-cancelled", () => {
                resolve(false);
            })

            autoUpdater.checkForUpdates().then(result => {
                if (result == null)
                    resolve(false);
            }).catch(() => reject());
            setTimeout( () => reject(false), 1000 * 30 );
        });
    }

    public downloadAndInstallUpdate() {
        return new Promise((resolve, reject) => {
            try {
                autoUpdater.on('update-downloaded', (ev) => {
                    // Wait 5 seconds, then quit and install
                    // In your application, you don't need to wait 5 seconds.
                    // You could call autoUpdater.quitAndInstall(); immediately
                    resolve(true);
                    setTimeout(() => autoUpdater.quitAndInstall(true, true), 5000);
                });
                autoUpdater.downloadUpdate().catch(() => reject());
            } catch (e) {
                reject();
            }
        })
    }
}